﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class GameUI : MonoBehaviour {

	private ShowPanels showPanels;

	private PlayMusic playMusic;

	public Text CountDownLabel;

	[SerializeField] private GameObject[] playerSprite = new GameObject[]{};

	// Use this for initialization
	void Awake ()
	{
		showPanels = GetComponent<ShowPanels> ();

		playMusic = GetComponent<PlayMusic> ();
	}

	public void QuitToMenu()
	{
		showPanels.HidePausePanel ();
		showPanels.HideEndGame ();
		SceneManager.LoadScene (1);
		showPanels.ShowMenu ();
	}

	public void RestartLevel()
	{
		showPanels.HidePausePanel ();
		showPanels.HideEndGame ();
		SceneManager.LoadScene (2);
		showPanels.ShowCountDownPanel ();
		GameController.Instance.PrepareGame ();
	}

	public void ChangeCountDownText(string newText){
		CountDownLabel.text = newText;
	}

	public void HideCountDownPanel()
	{
		showPanels.HideCountDownPanel ();
	}

	public void DoPause()
	{
		//call the ShowPausePanel function of the ShowPanels script
		showPanels.ShowPausePanel ();
	}


	public void UnPause()
	{
		//call the HidePausePanel function of the ShowPanels script
		showPanels.HidePausePanel ();
		GameController.Instance.ResumeGame ();
	}

	public Text VictoryPlayerText;

	public void GameOver(int player)
	{
		showPanels.ShowEndGame ();

		VictoryPlayerText.text = "Player " + (player + 1) + " won!";

		for (int i = 0; i < 8; i++)
		{
			if (i == player)
			{
				playerSprite [i].SetActive (true);
			}
			else
			{
				playerSprite [i].SetActive (false);
			}
		}
	}

	public void FadeUp()
	{
		playMusic.FadeUp (0);
	}
}
