﻿using System.Collections;
using UnityEngine.SceneManagement;
using UnityEngine;
using UnityEngine.UI;

public class GameController : MonoBehaviour {

	public static GameController Instance { get; set;} // para que qualquer classe encontre sem buscar componente

	private AudioSource audioSource;

	private GAME_STATE gameState = GAME_STATE.PRESS_TO_START;

	[SerializeField] private PlayerController[] playerControllers = new PlayerController[]{}; // lista de jogadores

	private int numberOfPlayers = 0; // contagem de quantos jogaroes entraram no jogo

	public AudioClip music;
	public AudioClip beep1;
	public AudioClip beep2;

	private float savedVolume;

	public KeyCode player1Control;
	public KeyCode player2Control;
	public KeyCode player3Control;
	public KeyCode player4Control;
	public KeyCode player5Control;
	public KeyCode player6Control;
	public KeyCode player7Control;
	public KeyCode player8Control;

	private GameUI gameUI;

	void Awake()
	{
		Instance = this;
		numberOfPlayers = PlayerPrefs.GetInt ("numberOfPlayers", 2);

		for (int i = 0; i < numberOfPlayers; i++) {
			playerControllers[i].EnableLable();
		}

		player1Control = (KeyCode)PlayerPrefs.GetInt ("Player1Control");
		player2Control = (KeyCode)PlayerPrefs.GetInt ("Player2Control");
		player3Control = (KeyCode)PlayerPrefs.GetInt ("Player3Control");
		player4Control = (KeyCode)PlayerPrefs.GetInt ("Player4Control");
		player5Control = (KeyCode)PlayerPrefs.GetInt ("Player5Control");
		player6Control = (KeyCode)PlayerPrefs.GetInt ("Player6Control");
		player7Control = (KeyCode)PlayerPrefs.GetInt ("Player7Control");
		player8Control = (KeyCode)PlayerPrefs.GetInt ("Player8Control");

		gameUI = GameObject.Find ("UI").GetComponent<GameUI> ();
		audioSource = gameUI.gameObject.GetComponent<AudioSource> ();
		audioSource.clip = null;
	}
		
	public void Start(){
		gameState = GAME_STATE.PRESS_TO_START;
		gameUI.ChangeCountDownText("Press Space to Start");
		Time.timeScale = 1;
		gameUI.FadeUp ();
		savedVolume = audioSource.volume;
	}

	// Update is called once per frame
	void Update () {

		switch (gameState) 
		{
			case GAME_STATE.PRESS_TO_START:
				if (Input.GetKeyDown (KeyCode.Space))
				{
					// sequencia de contagem
					//StartGame();
					gameUI.ChangeCountDownText("3");
					StartCoroutine (Count2 ());
					audioSource.PlayOneShot (beep1);
					gameState = GAME_STATE.GAME;
				}

				break;

			case GAME_STATE.GAME:
				if (Input.GetKeyDown (KeyCode.Escape))
				{
					gameState = GAME_STATE.PAUSE;
					Time.timeScale = 0;
					gameUI.DoPause ();
					savedVolume = audioSource.volume;
					audioSource.volume = savedVolume * 0.5f;
				}

				int playersAlive = numberOfPlayers;
				int lastAlive = 0;

				for (int i = 0; i < numberOfPlayers; i++) {
					if (playerControllers [i].isDead())
					{
						playersAlive--;
					} else
					{
						lastAlive = i;
					}
				}

				if(playersAlive == 1)
				{
					GameOver (lastAlive);
					gameState = GAME_STATE.END_GAME;
				}

				break;

			case GAME_STATE.PAUSE:
				if (Input.GetKeyDown (KeyCode.Escape))
				{
					ResumeGame ();
				}

				break;
			
			case GAME_STATE.END_GAME:
				break;
		}
	}

	IEnumerator Count2()
	{
		yield return new WaitForSeconds (1);
		gameUI.ChangeCountDownText("2");
		audioSource.PlayOneShot (beep1);
		StartCoroutine (Count1 ());
	}

	IEnumerator Count1()
	{
		yield return new WaitForSeconds (1);
		gameUI.ChangeCountDownText("1");
		audioSource.PlayOneShot (beep1);
		StartCoroutine (Go ());
	}

	IEnumerator Go()
	{
		yield return new WaitForSeconds (1);
		gameUI.ChangeCountDownText("GO");
		audioSource.PlayOneShot (beep2);
		StartCoroutine (StartGame ());
	}

	IEnumerator StartGame()
	{
		yield return new WaitForSeconds (.5f);

		for (int i = 0; i < numberOfPlayers; i++) {
			playerControllers[i].InstantiateCharacter ();
		}

		audioSource.clip = music;
		audioSource.Play ();

		gameUI.HideCountDownPanel ();
	}

	public void ResumeGame()
	{
		gameState = GAME_STATE.GAME;
		Time.timeScale = 1;
		if (audioSource.volume == savedVolume * 0.5f)
		{
			audioSource.volume = savedVolume;
		}
	}

	public void GameOver(int player)
	{
		gameUI.GameOver (player);
		Time.timeScale = 0;

		savedVolume = audioSource.volume;
		audioSource.volume = savedVolume * 0.5f;
	}

	public void PrepareGame()
	{
		audioSource.clip = null;
		audioSource.Pause ();
		audioSource.volume = savedVolume;
	}
}
