﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GroundTile : MonoBehaviour {

	public GroundTile leftTile; // o tile a esquerda

	public GroundTile rightTile; // o tile a direita

	public float time = 0.05f; // o tempo de delay da wave, quanto maior mais lenta

	public float fallVelocity = 0.5f; // velocidade que os tiles descem

	public float forceDecay = 0.97f; // quanto % da forca passa adiante

	public float minimumForceSpread = 0.3f; // forca minima para ela ser passada adiante

	public bool bounceOnEdge = false; // a forca bate e volta nas bordas?

	private Color myColor = Color.white; // a cor deste tile

	private float maxForce;

	private int direction = 0;

	// Use this for initialization
	void Start () {
		
	}
	
	// Update is called once per frame
	void LateUpdate ()
	{
		if (transform.localPosition.y > 0) { // se os tiles estiverem erquidos
			transform.localPosition = transform.localPosition + new Vector3 (0, -fallVelocity, 0); // movem pra baixo
			GetComponent<SpriteRenderer> ().color =  new Color (myColor.r , myColor.g, myColor.b, transform.localPosition.y / maxForce * 0.5f); // altera a cor conforme diminui

			if (transform.localPosition.y <= 0) { // se passou do nivel do chao, nivela tudo
				transform.localPosition = new Vector3 (transform.localPosition.x, 0, transform.localPosition.y);
				GetComponent<SpriteRenderer> ().color = new Color (1, 1, 1);

				direction = 0;
			}
		}
	}

	private void setColor(Color newColor) // muda a cor atual do tile
	{
		myColor = newColor;
		GetComponent<SpriteRenderer> ().color = myColor;
	}

	public void forceFromLeft(float force, Color newColor) // forca vindo da esquerda
	{
		direction = 1;

		setColor (newColor);

		if (force > maxForce) { // salva a forca maxima para calcular cor depois
			maxForce = force;
		}

		transform.localPosition = transform.localPosition + new Vector3 (0, force, 0);

		if (force > minimumForceSpread){ // se a forca for maior que um limite minimo
			if (rightTile) {  // se existir vizinho a direita passa ela adiante
				StartCoroutine (passForceRight (force * forceDecay, newColor)); // mas com menor intensidade
			}
			else if (bounceOnEdge && leftTile) { // se tiver bounce edge verifica se tem vizinho a esquerda e repassa
				StartCoroutine( passForceLeft (force * forceDecay, newColor));
			}
		}
	}

	public void forceFromRight(float force, Color newColor) // mesmo que a funcao de cima mas pro outro lado
	{
		direction = -1;

		setColor (newColor);

		if (force > maxForce) {
			maxForce = force;
		}

		transform.localPosition = transform.localPosition + new Vector3 (0, force, 0);

		if (force > minimumForceSpread)
		{
			if (leftTile) {
				StartCoroutine (passForceLeft (force * forceDecay, newColor));
			} else if (bounceOnEdge && rightTile) {
				StartCoroutine( passForceRight (force * forceDecay, newColor));
			}
		}
	}

	IEnumerator passForceRight(float force, Color newColor) // passa a forca da wave pra direita
	{
		yield return new WaitForSeconds(time);

		rightTile.forceFromLeft (force, newColor);
	}

	IEnumerator passForceLeft(float force, Color newColor) // passa a forca da wave pra esquerda
	{
		yield return new WaitForSeconds(time);

		leftTile.forceFromRight (force, newColor);
	}

	public int getDirection() {
		return direction;
	}
}
