﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerMove : MonoBehaviour {

	private PlayerSelect playerSelect;

	private AnimController playerAnim; // controlador de anim do player

	[SerializeField] private Transform anim; // visual do personagem
	
	public PlayerController playerController { get; set;} // controle do player para passar quando o character morre por cair no burco

	[SerializeField] private float walkSpeed; // velocidde de moviento do personagem
	public float jumpForce; // foca do pulo
	public float stompMoveCooldown= 1f;
	public int numberOfCumulativeWaves = 3;
	public float delayBetweenCumulativeWaves = 0.03f;
	public float waveRelativeForce = 1.5f;

	private AudioSource audioSource;

	public AudioClip thud;
	public AudioClip jump;

	public ParticleSystem stompParticle;

	private Rigidbody2D myBody; // corpo do personagem

	private bool isGrounded = false; // verificando se existe contato com o solo
	private bool canDoubleJump = false; // verifica se e possivel o personagem saltar mais uma vez
	private bool canStomp = false; // verifica se e possivel usar o stomp
	private  bool justTouchedGround = false; // se acabou de tocar o chão
	private bool stompWave = false; // já executou a onda do stomp

	private bool stomped = false; // verifica se o player usou stomp

	private float savedWalkSpeed; // velocidade antes de parar para o stomp

	public Color waveColor { get; set;} // cor da onda que sera gerada pelo personagem
	
	void Awake () {

		playerSelect = GetComponent<PlayerSelect> ();

		myBody = GetComponent<Rigidbody2D> (); // achando o body do personagem
		
		playerAnim = GetComponent<AnimController> (); // script que controla a animacao do personagem

		audioSource = GameObject.Find ("UI").GetComponent<AudioSource> ();
	}

	void Start()
	{
		if (myBody.transform.position.x > 0) {
			ChangeDirection ();
			canStomp = false;
		}
		canDoubleJump = true;
	}

	void Update(){
	
		if (playerSelect.Pressed() == true) { 

			if (isGrounded) { // se em contato com o solo
				StartCoroutine(delayedNotGrounded()); // checagem atrasada para contato com solo
				Jump (); // pula
			} else { // se nao
				if (canDoubleJump) {
					ChangeDirection (); // inverter a direcao do personagem
					Jump (); // mmaisum salto
					canDoubleJump = false; // para que o personagem nao possa mais saltar
				} else if (canStomp && !stomped) { // Se pode dar stomp e ainda náo deu
					savedWalkSpeed = walkSpeed; // salva o movimento atual
					walkSpeed = 0; // anula o movimento
					myBody.Sleep (); // para tudo
					myBody.AddForce(- myBody.transform.up * jumpForce); // cai com force
					stomped = true; // anota que ja deu stomp
					playerAnim.StompAnim (true); // mandando tocar a anmacao de salto
					StartCoroutine (continueWalking (stompMoveCooldown)); // conta o tempo para voltar a andar
				}
			}
		}

		playerAnim.SetVerticalSpeed (myBody.velocity.y);
		
		// verificado se o character caiu no buraco e morreu
		if(transform.position.y < -15f){
			Destroy (gameObject);
			playerController.Damage ();
		}

		// limitando para que o personagem nao entre dentro do solo
		if(myBody.position.x > -20f && transform.position.x < 20f){
			if (myBody.position.y < -4.7f) {
				myBody.position += Vector2.up * Time.deltaTime * 10;
			}
		}
	}
	
	void FixedUpdate () {

		myBody.transform.position += Time.deltaTime * walkSpeed * myBody.transform.right; // aplicando sempre a movimentacao do persoanegm

	}

	void ChangeDirection(){
		walkSpeed = walkSpeed * -1; // invertendo a direcao do movimento
		canStomp = true; // se deu double jum, pode dar stomp
		 // rotacionando o objeto que da o visual do personagem
		Vector3 rot = anim.eulerAngles;
		rot.y += 180;
		anim.eulerAngles = rot;
	}

	void Jump(){

		Vector2 bodyVelocity = new Vector2(0f,0f);

		// verificando se a velocidade vertical do personagem e positiva ou negativa

		/*if (myBody.velocity.y > 0) { // se maior do que zero 
			bodyVelocity = new Vector2 (myBody.velocity.x, 0f); // anular a veocidade vertical do personagem
		}else if (myBody.velocity.y < 0) { // se menor que zero
			bodyVelocity = myBody.velocity; // aproveitar a velocidade vertical do personagem
		}*/

		bodyVelocity = new Vector2 (myBody.velocity.x, 0f); // anular a veocidade vertical do personagem

		myBody.Sleep (); // sempre anular a velocidade 

		myBody.velocity = bodyVelocity; // atribuir a velocidade calculada com base no valor calculado

		myBody.AddForce (myBody.transform.up * jumpForce); // aplicar o salto
		
		playerAnim.JumpAnim (true); // mandando tocar a anmacao de salto

		audioSource.PlayOneShot (jump);
	}



	// verificando se o personagem esta em contato com o solo 
	void OnCollisionEnter2D(Collision2D coll){
		if (coll.gameObject.tag == "Ground" && !isGrounded && !justTouchedGround) { // se nao ta no chao e nao acabaou de cair
			playerAnim.JumpAnim (false); // mandando tocar a anmacao de salto
			justTouchedGround = true; // acabou de cair
			StartCoroutine (justTouchedGroundEnd ()); // controle anti-bug muito louco
			if (!stomped) { // se nao deu stomp reseta as variaveis de pulo
				isGrounded = true;
				canDoubleJump = true;
				canStomp = false;
			} else if (stomped && !stompWave) { // se deu stomp e ainda nao fez a wave
				stompWave = true; // anota que fez a wave
				GroundTile sourceTile = coll.gameObject.GetComponent<GroundTile> (); // pega o tile em que pousou e executa as waves

				ShakeCamera.Instance.DoShake (); // chando na camera a funcao que faz ela balancar

				audioSource.PlayOneShot (thud);

				//stompParticle.main.startColor = Color.yellow;

				Instantiate (stompParticle, transform.position, Quaternion.identity);

				for (int i = 0; i < numberOfCumulativeWaves; i++) {
					StartCoroutine (releaseWave(sourceTile, i * delayBetweenCumulativeWaves, coll.relativeVelocity.magnitude * (0.1f * waveRelativeForce)));
				}
			}
		}

		if (coll.gameObject.tag == "Ground" && coll.gameObject.GetComponent<GroundTile> ().getDirection () != 0 && !justTouchedGround) {
			
			justTouchedGround = true; // acabou de cair
			StartCoroutine (justTouchedGroundEnd ()); 
			isGrounded = false;
			canDoubleJump = false;
			canStomp = false;
			if (coll.gameObject.GetComponent<GroundTile> ().getDirection () > 0) {
				myBody.Sleep ();
				myBody.AddForce ((transform.up + transform.right).normalized * 10, ForceMode2D.Impulse);
			} else {
				myBody.Sleep ();
				myBody.AddForce ((transform.up + -transform.right).normalized * 10, ForceMode2D.Impulse);
			}
		}
	}

	IEnumerator continueWalking(float time) // continua andando apos stomp
	{
		yield return new WaitForSeconds(time);

		// reseta as variavis de controle pra pulo
		isGrounded = true;
		canDoubleJump = true;
		canStomp = false;
		stomped = false;
		stompWave = false;

		playerAnim.StompAnim (false); // mandando tocar a anmacao de salto
		
		// volta a andar para a direcao salva
		walkSpeed = savedWalkSpeed;
	}

	IEnumerator delayedNotGrounded() // controle para bug toque no chao
	{
		yield return new WaitForSeconds(0.02f);
		isGrounded = false;
	}

	IEnumerator justTouchedGroundEnd() // controle pra outro bug
	{
		yield return new WaitForSeconds(0.02f);
		justTouchedGround = false;
	}

	IEnumerator releaseWave(GroundTile sourceTile, float delay, float force) //executa as waves para os 2 lados
	{
		yield return new WaitForSeconds(delay);

		if(sourceTile.leftTile)	{ // sequencia louca de ifs pra nao acessar valor null
			if (sourceTile.leftTile.leftTile) {
				if (sourceTile.leftTile.leftTile.leftTile) {
					sourceTile.leftTile.leftTile.leftTile.forceFromRight (force, GetComponent<PlayerMove> ().waveColor); // executa a wave com a cor do player
				}
			}
		}
		
		if (sourceTile.rightTile) {
			if (sourceTile.rightTile.rightTile) {
				if (sourceTile.rightTile.rightTile.rightTile) {
					sourceTile.rightTile.rightTile.rightTile.forceFromLeft (force, GetComponent<PlayerMove> ().waveColor);
				}
			}
		}
	}
}
